package com.example.jairo.wristcare.modelo;

import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.Build;
import java.text.SimpleDateFormat;

import com.example.jairo.wristcare.modelo.ProgramsContract.SessionEntry;

import java.util.Date;

import static com.example.jairo.wristcare.modelo.ProgramsContract.ProgramEntry;

/**
 * Manejador de la base de datos
 */
public class ProgramsDbHelper extends SQLiteOpenHelper {
    public static final int DATABASE_VERSION = 1;
    public static final String DATABASE_NAME = "Programs.db";

    public ProgramsDbHelper(Context context) {
        super(context, DATABASE_NAME, null, DATABASE_VERSION);
    }

    @Override
    public void onOpen(SQLiteDatabase db) {
        super.onOpen(db);
        if (!db.isReadOnly()) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN) {
                db.setForeignKeyConstraintsEnabled(true);
            } else {
                db.execSQL("PRAGMA foreign_keys=ON");
            }
        }
    }

    @Override
    public void onCreate(SQLiteDatabase db) {
        db.execSQL("CREATE TABLE " + ProgramEntry.TABLE_NAME + " ("
                + ProgramEntry._ID + " INTEGER PRIMARY KEY AUTOINCREMENT,"
                + ProgramEntry.NAME + " TEXT NOT NULL,"
                + ProgramEntry.FCREATION + " TEXT NOT NULL,"
                + ProgramEntry.FCLOSE + " TEXT,"
                + ProgramEntry.OPEN + " INTEGER NOT NULL,"
                + ProgramEntry.INITFLEX + " REAL,"
                + ProgramEntry.INITEXT + " REAL,"
                + ProgramEntry.INITABD + " REAL,"
                + ProgramEntry.INITADU + " REAL,"
                + ProgramEntry.INITSUP + " REAL,"
                + ProgramEntry.INITPRO + " REAL,"
                + "UNIQUE (" + ProgramEntry.NAME + "))");

        db.execSQL("CREATE TABLE " + SessionEntry.TABLE_NAME + " ("
                + SessionEntry._ID + " INTEGER PRIMARY KEY AUTOINCREMENT,"
                + SessionEntry.FECHA + " TEXT NOT NULL,"
                + SessionEntry.ANFLEX + " REAL,"
                + SessionEntry.ANEXT + " REAL,"
                + SessionEntry.ANABD + " REAL,"
                + SessionEntry.ANADU + " REAL,"
                + SessionEntry.ANPRO + " REAL,"
                + SessionEntry.ANSUP + " REAL,"
                + SessionEntry.ID_PROGRAM + " INTEGER NOT NULL,"
                + "FOREIGN KEY (" + SessionEntry.ID_PROGRAM + ") REFERENCES "+  ProgramEntry.TABLE_NAME +"("+ ProgramEntry._ID +") ON DELETE CASCADE )");

    }


    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        db.execSQL("DROP TABLE IF EXISTS " + ProgramEntry.TABLE_NAME);
        db.execSQL("DROP TABLE IF EXISTS " + SessionEntry.TABLE_NAME);;

        onCreate(db);
    }

    public long saveProgram(Program program) {
        SQLiteDatabase sqLiteDatabase = getWritableDatabase();

        return sqLiteDatabase.insert(
                ProgramEntry.TABLE_NAME,
                null,
                program.toContentValues());

    }

    public Cursor getFlexExtByName(String programName) {
        return getReadableDatabase().rawQuery("SELECT session.fecha, session.anflex, session.anext "+
                                              "FROM program, session " +
                                              "WHERE (program._id = session.idprogram) AND program.name ='" + programName + "' AND (session.anflex IS NOT NULL OR session.anext IS NOT NULL)", null);
    }

    public Cursor getAbdAduByName(String programName) {
        return getReadableDatabase().rawQuery("SELECT session.fecha, session.anabd, session.anadu "+
                                              "FROM program, session " +
                                              "WHERE (program._id = session.idprogram) AND program.name = '" + programName + "' AND (session.anabd IS NOT NULL OR session.anadu IS NOT NULL)", null);
    }

    public Cursor getSupPronByName(String programName) {
        return getReadableDatabase().rawQuery("SELECT session.fecha, session.ansup, session.anpro "+
                                              "FROM program, session " +
                                              "WHERE (program._id = session.idprogram) AND program.name = '" + programName + "' AND (session.ansup IS NOT NULL OR session.anpro IS NOT NULL)", null);
    }

    public Cursor getAllSessionsByName(String programName){
        return getReadableDatabase()
                .rawQuery("SELECT session.fecha FROM session WHERE session.idprogram = (SELECT program._id FROM program WHERE program.name = '" + programName + "') ORDER BY session._id DESC", null);
    }

    public Cursor getFlexExtTodayByName(String programName) {
        return getReadableDatabase().rawQuery("SELECT session.anflex, session.anext "+
                "FROM program, session " +
                "WHERE (program._id = session.idprogram) AND program.name ='" + programName + "' AND (session.fecha = '" + formatDateToday() + "')", null);
    }

    public Cursor getAbdAduTodayByName(String programName) {
        return getReadableDatabase().rawQuery("SELECT session.anabd, session.anadu "+
                "FROM program, session " +
                "WHERE (program._id = session.idprogram) AND program.name ='" + programName + "' AND (session.fecha = '" + formatDateToday() + "')", null);
    }

    public Cursor getSupProTodayByName(String programName) {
        return getReadableDatabase().rawQuery("SELECT session.ansup, session.anpro "+
                "FROM program, session " +
                "WHERE (program._id = session.idprogram) AND program.name ='" + programName + "' AND (session.fecha = '" + formatDateToday() + "')", null);
    }

    public Cursor getAnglesLastSessionByName(String programName) {
        return getReadableDatabase().rawQuery("SELECT session.fecha, session.anflex, session.anext, session.anabd, session.anadu, session.ansup, session.anpro, session.idprogram "+
                "FROM program, session " +
                "WHERE (program._id = session.idprogram) AND program.name ='" + programName + "' ORDER BY session._id DESC", null);
    }

    public Cursor getIdSessionTodayByName(String programName) {
        return getReadableDatabase().rawQuery("SELECT session._id "+
                "FROM program, session " +
                "WHERE (program._id = session.idprogram) AND program.name ='" + programName + "' AND (session.fecha = '" + formatDateToday() + "')", null);
    }

    public Cursor getAllPrograms() {
        return getReadableDatabase()
                .query(
                        ProgramEntry.TABLE_NAME,
                        null,
                        null,
                        null,
                        null,
                        null,
                        null);
    }

    public Cursor getProgramByName(String programName) {
        Cursor c = getReadableDatabase().query(
                ProgramEntry.TABLE_NAME,
                null,
                ProgramEntry.NAME + " LIKE ?",
                new String[]{programName},
                null,
                null,
                null);
        return c;
    }

    public int deleteProgram(String programName) {
        return getWritableDatabase().delete(
                ProgramEntry.TABLE_NAME,
                ProgramEntry.NAME + " LIKE ?",
                new String[]{programName});
    }

    public int updateProgram(Program program, String programName) {
        return getWritableDatabase().update(
                ProgramEntry.TABLE_NAME,
                program.toContentValues(),
                ProgramEntry.NAME + " LIKE ?",
                new String[]{programName}
        );
    }

    public int updateSession(Session session, int sessionId) {
        return getWritableDatabase().update(
                SessionEntry.TABLE_NAME,
                session.toContentValues(),
                SessionEntry._ID + " LIKE ?",
                new String[]{String.valueOf(sessionId)}
        );
    }

    public Cursor getSessionById(int session){
        Cursor c = getReadableDatabase().query(
                SessionEntry.TABLE_NAME,
                null,
                SessionEntry._ID + " LIKE ?",
                new String[]{String.valueOf(session)},
                null,
                null,
                null);
        return c;
    }

    public long saveSession(Session session) {
        SQLiteDatabase sqLiteDatabase = getWritableDatabase();

        return sqLiteDatabase.insert(
                SessionEntry.TABLE_NAME,
                null,
                session.toContentValues());

    }

    private String formatDateToday(){
        return new SimpleDateFormat("yyyy-MM-dd").format(new Date());
    }
}