package com.example.jairo.wristcare.presentador;

import android.content.Context;
import android.database.Cursor;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;

import com.example.jairo.wristcare.vista.AbdAduExerciseFragmentView;
import com.example.jairo.wristcare.modelo.ProgramsDbHelper;
import com.example.jairo.wristcare.modelo.Session;
import com.example.jairo.wristcare.util.SensorFusion;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Created by Jaito on 09/06/2017.
 */
public class AbdAduExerciseFragmentPresenter implements SensorEventListener {
    private ProgramsDbHelper mProgramsDbHelper;
    private AbdAduExerciseFragmentView mAbdAduExerciseFragmentView;
    private SensorFusion sensorFusion;
    private SensorManager sensorManager = null;
    private DecimalFormat d;

    public AbdAduExerciseFragmentPresenter(Context context, AbdAduExerciseFragmentView view, SensorManager sensorM){
        mProgramsDbHelper = new ProgramsDbHelper(context);
        mAbdAduExerciseFragmentView = view;
        sensorManager = sensorM;

        registerSensorManagerListeners();

        DecimalFormatSymbols symbols = DecimalFormatSymbols.getInstance();
        symbols.setDecimalSeparator('.');
        d = new DecimalFormat("#.##", symbols);
        d.setMaximumFractionDigits(2);
        d.setMinimumFractionDigits(2);

        sensorFusion = new SensorFusion();
        sensorFusion.setMode(SensorFusion.Mode.FUSION);
    }

    public void updateSession(int session, double abd, double adu){
        Cursor c = mProgramsDbHelper.getSessionById(session);
        if(c != null && c.moveToNext()) {
            Session s = new Session(c);
            if(s.getAnFlex() == 0.0) {
                s.setAnFlex(Double.NaN);
            }
            if(s.getAnExt() == 0.0) {
                s.setAnExt(Double.NaN);
            }
            s.setAnAbd(abd);
            s.setAnAdu(adu);
            if(s.getAnSup() == 0.0) {
                s.setAnSup(Double.NaN);
            }
            if(s.getAnPro() == 0.0) {
                s.setAnPro(Double.NaN);
            }
            mProgramsDbHelper.updateSession(s, session);

            c.close();
        }
    }

    public void createSession(String programName, double abd, double adu) {
        int idProgram;
        String fecha = new SimpleDateFormat("yyyy-MM-dd").format(new Date());
        Cursor c = mProgramsDbHelper.getProgramByName(programName);
        if(c != null && c.moveToNext()) {
            idProgram = c.getInt(0);
            Session s = new Session(fecha, Double.NaN, Double.NaN, abd, adu, Double.NaN, Double.NaN, idProgram);
            mProgramsDbHelper.saveSession(s);
        }
    }

    public boolean checkRoll(){
        return ((Math.abs(sensorFusion.getRoll()) > 80) && (Math.abs(sensorFusion.getRoll()) < 100));
    }

    public void getInitDegree(){
        mAbdAduExerciseFragmentView.setInitDegree(sensorFusion.getPitch());
    }

    public void getFinalDegree(){
        mAbdAduExerciseFragmentView.setFinalDegree(sensorFusion.getPitch());
    }

    public void calculateDegree(double init, double end){
        mAbdAduExerciseFragmentView.setCalculatedDegree(String.valueOf(d.format(calcDegree(init, end))));
    }

    public void calculateDiffLastSession(double init, double end, String programName, int screen){
        String result = "0.0";
        double last;
        boolean flag = false;
        Cursor c = mProgramsDbHelper.getAnglesLastSessionByName(programName);
        if(c != null) {
            while (c.moveToNext() && !flag) {
                Session s = new Session(c);
                if (s.getAnAbd() != 0.0 || s.getAnAdu() != 0.0) {
                    flag = true;
                    if (screen == 0) {
                        last = s.getAnAbd();
                    } else {
                        last = s.getAnAdu();
                    }
                    result = String.valueOf(d.format((calcDegree(init, end)) - last));

                }
            }
            c.close();
        } else {
            result = d.format(Double.valueOf(calcDegree(init, end)));
        }
        mAbdAduExerciseFragmentView.setDiffLastSessionDegree(result);
    }

    public void registerSensorManagerListeners() {
        sensorManager.registerListener(this,
                sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER),
                SensorManager.SENSOR_DELAY_FASTEST);

        sensorManager.registerListener(this,
                sensorManager.getDefaultSensor(Sensor.TYPE_GYROSCOPE),
                SensorManager.SENSOR_DELAY_FASTEST);

        sensorManager.registerListener(this,
                sensorManager.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD),
                SensorManager.SENSOR_DELAY_FASTEST);
    }

    public void updateOrCreateSession(String programName, double abd, double adu){
        if(checkSessionToday(programName)){
            updateSession(getIdSession(programName), abd, adu);
        } else {
            createSession(programName, abd, adu);
        }
    }

    public void unregisterListener(){
        sensorManager.unregisterListener(this);
    }

    @Override
    public void onAccuracyChanged(Sensor sensor, int accuracy) {
    }

    @Override
    public void onSensorChanged(SensorEvent event) {
        switch (event.sensor.getType()) {
            case Sensor.TYPE_ACCELEROMETER:
                sensorFusion.setAccel(event.values);
                sensorFusion.calculateAccMagOrientation();
                break;

            case Sensor.TYPE_GYROSCOPE:
                sensorFusion.gyroFunction(event);
                break;

            case Sensor.TYPE_MAGNETIC_FIELD:
                sensorFusion.setMagnet(event.values);
                break;
        }
    }

    private int getIdSession(String programName){
        Cursor c = mProgramsDbHelper.getIdSessionTodayByName(programName);
        if(c != null && c.moveToNext()){
            return c.getInt(0);
        } else {
            return -1;
        }
    }

    private boolean checkSessionToday(String programName){
        Cursor c = mProgramsDbHelper.getAbdAduTodayByName(programName);
        return (c != null && c.moveToNext());
    }

    private double calcDegree(double init, double end){
        double result;

        if((init > 0.0 && end > 0.0) || (init < 0.0 && end < 0.0)){
            result = Math.abs(end - init);
        } else if (init > 0.0 && end < 0.0){
            result = Math.min((180.0 - Math.abs(end)) + 180.0 - init,(Math.abs(end) + init));
        } else {
            result = Math.min((180.0 - Math.abs(init)) + 180.0 - end,(Math.abs(init) + end));
        }

        return result;
    }
}
