package com.example.jairo.wristcare.presentador;

import android.content.Context;
import android.database.Cursor;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;

import com.example.jairo.wristcare.vista.FlexExtExerciseFragmentView;
import com.example.jairo.wristcare.modelo.ProgramsDbHelper;
import com.example.jairo.wristcare.modelo.Session;
import com.example.jairo.wristcare.util.SensorFusion;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Created by Jaito on 09/06/2017.
 */
public class FlexExtExerciseFragmentPresenter implements SensorEventListener{

    private ProgramsDbHelper mProgramsDbHelper;
    private FlexExtExerciseFragmentView mFlexExtExerciseFragmentView;
    private SensorFusion sensorFusion;
    private SensorManager sensorManager = null;
    private DecimalFormat d;

    public FlexExtExerciseFragmentPresenter(Context context,
                                            FlexExtExerciseFragmentView view,
                                            SensorManager sensorM){
        mProgramsDbHelper = new ProgramsDbHelper(context);
        mFlexExtExerciseFragmentView = view;
        sensorManager = sensorM;

        registerSensorManagerListeners();

        DecimalFormatSymbols symbols = DecimalFormatSymbols.getInstance();
        symbols.setDecimalSeparator('.');
        d = new DecimalFormat("#.##", symbols);
        d.setMaximumFractionDigits(2);
        d.setMinimumFractionDigits(2);

        sensorFusion = new SensorFusion();
        sensorFusion.setMode(SensorFusion.Mode.FUSION);
    }

    public void updateSession(int session, double flex, double ext){
        Cursor c = mProgramsDbHelper.getSessionById(session);
        if(c != null && c.moveToNext()) {
            Session s = new Session(c);
            s.setAnFlex(flex);
            s.setAnExt(ext);
            if(s.getAnAbd() == 0.0) {
                s.setAnAbd(Double.NaN);
            }
            if(s.getAnAdu() == 0.0) {
                s.setAnAdu(Double.NaN);
            }
            if(s.getAnSup() == 0.0) {
                s.setAnSup(Double.NaN);
            }
            if(s.getAnPro() == 0.0) {
                s.setAnPro(Double.NaN);
            }
            mProgramsDbHelper.updateSession(s, session);

            c.close();
        }
    }

    public void createSession(String programName, double flex, double ext) {
        int idProgram;
        String fecha = new SimpleDateFormat("yyyy-MM-dd").format(new Date());
        Cursor c = mProgramsDbHelper.getProgramByName(programName);
        if (c != null && c.moveToNext()) {
            idProgram = c.getInt(0);
            Session s = new Session(fecha, flex, ext, Double.NaN, Double.NaN, Double.NaN, Double.NaN, idProgram);
            mProgramsDbHelper.saveSession(s);

            c.close();
        }
    }

    public boolean checkRoll(){
        return ((Math.abs(sensorFusion.getRoll()) > 80) && (Math.abs(sensorFusion.getRoll()) < 100));
    }

    public void getInitDegree(){
        mFlexExtExerciseFragmentView.setInitDegree(sensorFusion.getAzimuth());
    }

    public void getFinalDegree(){
        mFlexExtExerciseFragmentView.setFinalDegree(sensorFusion.getAzimuth());
    }

    public void calculateDegree(double init, double end){
        mFlexExtExerciseFragmentView.setCalculatedDegree(String.valueOf(d.format(calcDegree(init, end))));
    }

    public void calculateDiffLastSession(double init, double end, String programName, int screen){
        String result;
        double last;
        Cursor c = mProgramsDbHelper.getAnglesLastSessionByName(programName);
        if(c != null && c.moveToNext()){
            Session s = new Session(c);
            if(screen == 0){
                last = s.getAnFlex();
            } else {
                last = s.getAnExt();
            }
            result = String.valueOf(d.format((calcDegree(init, end)) - last));
            c.close();
        } else {
            result = d.format(Double.valueOf(calcDegree(init, end)));
        }
        mFlexExtExerciseFragmentView.setDiffLastSessionDegree(result);
    }

    public void registerSensorManagerListeners() {
        sensorManager.registerListener(this,
                sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER),
                SensorManager.SENSOR_DELAY_FASTEST);

        sensorManager.registerListener(this,
                sensorManager.getDefaultSensor(Sensor.TYPE_GYROSCOPE),
                SensorManager.SENSOR_DELAY_FASTEST);

        sensorManager.registerListener(this,
                sensorManager.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD),
                SensorManager.SENSOR_DELAY_FASTEST);
    }

    public void updateOrCreateSession(String programName, double flex, double ext){
        if(checkSessionToday(programName)){
            updateSession(getIdSession(programName), flex, ext);
        } else {
            createSession(programName, flex, ext);
        }
    }

    public void unregisterListener(){
        sensorManager.unregisterListener(this);
    }

    @Override
    public void onAccuracyChanged(Sensor sensor, int accuracy) {
    }

    @Override
    public void onSensorChanged(SensorEvent event) {
        switch (event.sensor.getType()) {
            case Sensor.TYPE_ACCELEROMETER:
                sensorFusion.setAccel(event.values);
                sensorFusion.calculateAccMagOrientation();
                break;

            case Sensor.TYPE_GYROSCOPE:
                sensorFusion.gyroFunction(event);
                break;

            case Sensor.TYPE_MAGNETIC_FIELD:
                sensorFusion.setMagnet(event.values);
                break;
        }
    }

    private int getIdSession(String programName){
        Cursor c = mProgramsDbHelper.getIdSessionTodayByName(programName);
        if(c != null && c.moveToNext()){
            return c.getInt(0);
        } else {
            return -1;
        }
    }

    private boolean checkSessionToday(String programName){
        Cursor c = mProgramsDbHelper.getFlexExtTodayByName(programName);
        return (c != null && c.moveToNext());
    }

    private double calcDegree(double init, double end){
        double result;

        if((init > 0.0 && end > 0.0) || (init < 0.0 && end < 0.0)){
            result = Math.abs(end - init);
        } else if (init > 0.0 && end < 0.0){
            result = Math.min((180.0 - Math.abs(end)) + 180.0 - init,(Math.abs(end) + init));
        } else {
            result = Math.min((180.0 - Math.abs(init)) + 180.0 - end,(Math.abs(init) + end));
        }

        return result;
    }
}
