package com.example.jairo.wristcare.presentador;

import android.content.Context;
import android.database.Cursor;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.util.Log;

import com.example.jairo.wristcare.vista.SupProExerciseFragmentView;
import com.example.jairo.wristcare.modelo.Program;
import com.example.jairo.wristcare.modelo.ProgramsDbHelper;
import com.example.jairo.wristcare.modelo.Session;
import com.example.jairo.wristcare.util.SensorFusion;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Created by Jaito on 09/06/2017.
 */
public class SupProExerciseFragmentPresenter implements SensorEventListener {
    private ProgramsDbHelper mProgramsDbHelper;
    private SupProExerciseFragmentView mSupProExerciseFragmentView;
    private SensorFusion sensorFusion;
    private SensorManager sensorManager = null;
    private DecimalFormat d;

    public SupProExerciseFragmentPresenter(Context context, SupProExerciseFragmentView view, SensorManager sensorM){
        mProgramsDbHelper = new ProgramsDbHelper(context);
        mSupProExerciseFragmentView = view;
        sensorManager = sensorM;

        registerSensorManagerListeners();

        DecimalFormatSymbols symbols = DecimalFormatSymbols.getInstance();
        symbols.setDecimalSeparator('.');
        d = new DecimalFormat("#.##", symbols);
        d.setMaximumFractionDigits(2);
        d.setMinimumFractionDigits(2);

        sensorFusion = new SensorFusion();
        sensorFusion.setMode(SensorFusion.Mode.FUSION);
    }

    public void updateSession(int session, double sup, double pro){
        Cursor c = mProgramsDbHelper.getSessionById(session);
        if(c != null && c.moveToNext()) {
            Session s = new Session(c);
            if(s.getAnFlex() == 0.0) {
                s.setAnFlex(Double.NaN);
            }
            if(s.getAnExt() == 0.0) {
                s.setAnExt(Double.NaN);
            }
            if(s.getAnAbd() == 0.0) {
                s.setAnAbd(Double.NaN);
            }
            if(s.getAnAdu() == 0.0) {
                s.setAnAdu(Double.NaN);
            }
            s.setAnSup(sup);
            s.setAnPro(pro);
            mProgramsDbHelper.updateSession(s, session);
            c.close();
        }
    }

    public void createSession(String programName, double sup, double pro) {
        int idProgram;
        String fecha = new SimpleDateFormat("yyyy-MM-dd").format(new Date());
        Cursor c = mProgramsDbHelper.getProgramByName(programName);
        if(c != null && c.moveToNext()) {
            idProgram = c.getInt(0);
            Session s = new Session(fecha, Double.NaN, Double.NaN, Double.NaN, Double.NaN, sup, pro, idProgram);
            mProgramsDbHelper.saveSession(s);

            c.close();
        }
    }

    public void addProgram(String programName, String date, int open,double flex, double ext, double abd, double adu, double sup, double pro){
        Program p = new Program(programName, date, null, open, flex, ext, abd, adu, sup, pro);
        mSupProExerciseFragmentView.showProgramsScreen(mProgramsDbHelper.saveProgram(p) > 0);
    }

    public boolean checkRoll(){
        return ((Math.abs(sensorFusion.getRoll()) > 80) && (Math.abs(sensorFusion.getRoll()) < 100));
    }

    public void getInitDegree(){
        mSupProExerciseFragmentView.setInitDegree(sensorFusion.getRoll());
    }

    public void getFinalDegree(){
        mSupProExerciseFragmentView.setFinalDegree(sensorFusion.getRoll());
    }

    public void calculateDegree(double init, double end){
        mSupProExerciseFragmentView.setCalculatedDegree(String.valueOf(d.format(calcDegree(init, end))));
    }

    public void calculateDiffLastSession(double init, double end, String programName, int screen){
        String result;
        double last;
        Cursor c = mProgramsDbHelper.getAnglesLastSessionByName(programName);
        if(c != null && c.moveToNext()){
            Session s = new Session(c);
            if(screen == 0){
                last = s.getAnSup();
            } else {
                last = s.getAnPro();
            }
            result = String.valueOf(d.format((calcDegree(init, end)) - last));
            c.close();
        } else {
            result = d.format(Double.valueOf(calcDegree(init, end)));
        }
        mSupProExerciseFragmentView.setDiffLastSessionDegree(result);
    }

    public void registerSensorManagerListeners() {
        sensorManager.registerListener(this,
                sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER),
                SensorManager.SENSOR_DELAY_FASTEST);

        sensorManager.registerListener(this,
                sensorManager.getDefaultSensor(Sensor.TYPE_GYROSCOPE),
                SensorManager.SENSOR_DELAY_FASTEST);

        sensorManager.registerListener(this,
                sensorManager.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD),
                SensorManager.SENSOR_DELAY_FASTEST);
    }

    public void updateOrCreateSession(String programName, double sup, double pro){
        if(checkSessionToday(programName)){
            updateSession(getIdSession(programName), sup, pro);
        } else {
            createSession(programName, sup, pro);
        }
    }

    public void unregisterListener(){
        sensorManager.unregisterListener(this);
    }

    @Override
    public void onAccuracyChanged(Sensor sensor, int accuracy) {
    }

    @Override
    public void onSensorChanged(SensorEvent event) {
        switch (event.sensor.getType()) {
            case Sensor.TYPE_ACCELEROMETER:
                sensorFusion.setAccel(event.values);
                sensorFusion.calculateAccMagOrientation();
                break;

            case Sensor.TYPE_GYROSCOPE:
                sensorFusion.gyroFunction(event);
                break;

            case Sensor.TYPE_MAGNETIC_FIELD:
                sensorFusion.setMagnet(event.values);
                break;
        }
    }

    private int getIdSession(String programName){
        Cursor c = mProgramsDbHelper.getIdSessionTodayByName(programName);
        if(c != null && c.moveToNext()){
            return c.getInt(0);
        } else {
            return -1;
        }
    }

    private boolean checkSessionToday(String programName){
        Cursor c = mProgramsDbHelper.getSupProTodayByName(programName);
        return (c != null && c.moveToNext());
    }

    private double calcDegree(double init, double end){
        double result;

        if((init > 0.0 && end > 0.0) || (init < 0.0 && end < 0.0)){
            result = Math.abs(end - init);
        } else if (init > 0.0 && end < 0.0){
            result = Math.min((180.0 - Math.abs(end)) + 180.0 - init,(Math.abs(end) + init));
        } else {
            result = Math.min((180.0 - Math.abs(init)) + 180.0 - end,(Math.abs(init) + end));
        }

        return result;
    }
}
