package com.example.jairo.wristcare.vista;


import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.TextInputEditText;
import android.support.design.widget.TextInputLayout;
import android.support.v4.app.Fragment;
import android.support.v7.app.AlertDialog;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.example.jairo.wristcare.presentador.AddProgramsPresenter;

import com.example.jairo.wristcare.R;


public class AddProgramFragment extends Fragment implements AddProgramsView {
    private static final String ARG_PROGRAM_NAME = "arg_program_name";
    public static final int REQUEST_INITIAL_EXERCISE = 1;

    private FloatingActionButton mSaveButton;
    private TextInputEditText mNameField;
    private TextInputLayout mNameLabel;

    private AddProgramsPresenter mAddProgramsPresenter;

    public AddProgramFragment() {
        // Required empty public constructor
    }

    public static AddProgramFragment newInstance(String programName) {
        AddProgramFragment fragment = new AddProgramFragment();
        Bundle args = new Bundle();
        args.putString(ARG_PROGRAM_NAME, programName);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mAddProgramsPresenter = new AddProgramsPresenter(getActivity(),this);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View root = inflater.inflate(R.layout.fragment_add_program, container, false);

        // Referencias UI
        mSaveButton = (FloatingActionButton) getActivity().findViewById(R.id.fab);
        mNameField = (TextInputEditText) root.findViewById(R.id.et_name);
        mNameLabel = (TextInputLayout) root.findViewById(R.id.til_name);

        // Eventos
        mSaveButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                addProgram();
            }
        });

        return root;
    }

    @Override
    public void checkName(Cursor c) {
        boolean error = false;

        String name = mNameField.getText().toString();

        if (TextUtils.isEmpty(name)) {
            mNameLabel.setError(getString(R.string.field_error));
            error = true;
        }

        if(c != null && c.moveToNext()){
            mNameLabel.setError(getString(R.string.unique_error));
            error = true;
            c.close();
        }


        if (!error) {
            showDialogReferenceExercise();
        }
    }

    private void addProgram() {
        mAddProgramsPresenter.getProgramByName(mNameField.getText().toString());
    }

    private void showDialogReferenceExercise(){
        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
        builder.setTitle(getString(R.string.dialog_title_reference_exercise));
        builder.setMessage(getString(R.string.dialog_message_reference_exercise));

        String positiveText = getString(R.string.ok_continuar);
        builder.setPositiveButton(positiveText,
                new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        Intent intent = new Intent(getActivity(), FlexExtExerciseActivity.class);
                        intent.putExtra("ARG_PROGRAM_NAME", mNameField.getText().toString());
                        startActivityForResult(intent, REQUEST_INITIAL_EXERCISE);
                    }
                });

        AlertDialog dialog = builder.create();

        dialog.show();
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (Activity.RESULT_OK == resultCode) {
            getActivity().setResult(Activity.RESULT_OK);
        } else {
            getActivity().setResult(Activity.RESULT_CANCELED);
        }

        getActivity().finish();
    }

}
