package com.example.jairo.wristcare.vista;

import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.TabLayout;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.view.ViewPager;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageView;
import android.widget.Toast;

import com.example.jairo.wristcare.R;
import com.example.jairo.wristcare.presentador.ProgramDetailActivityPresenter;

import java.util.ArrayList;
import java.util.List;

public class ProgramDetailActivity extends AppCompatActivity implements ProgramDetailActivityView {
    public static final String EXTRA_PROGRAM_NAME = "extra_program_name";
    public static final int REQUEST_EXERCISE_DONE = 1;
    public static String programName;
    public static boolean closed;

    private ProgramDetailActivityPresenter mPresenter;

    private ImageView mImageView;
    private FloatingActionButton fab;
    private Toolbar toolbar;
    private TabLayout tabLayout;
    private ViewPager viewPager;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_program_detail);

        mPresenter = new ProgramDetailActivityPresenter(this, this);

        toolbar = (Toolbar) findViewById(R.id.toolbarDetail);
        setSupportActionBar(toolbar);

        if(getSupportActionBar() != null) {
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        }

        programName = getIntent().getStringExtra(ProgramsActivity.EXTRA_PROGRAM_NAME);

        fab = (FloatingActionButton) findViewById(R.id.fabDetail);

        closed = false;

        mImageView = (ImageView) findViewById(R.id.imgBackgroundDetail);

        viewPager = (ViewPager) findViewById(R.id.viewpager);
        setupViewPager(viewPager);

        tabLayout = (TabLayout) findViewById(R.id.tabs);
        tabLayout.setupWithViewPager(viewPager);

        viewPager.addOnPageChangeListener(new ViewPager.OnPageChangeListener(){
            FloatingActionButton fab = (FloatingActionButton) findViewById(R.id.fabDetail);
            @Override
            public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {

            }

            @Override
            public void onPageSelected(int position) {
                if(position == 0) {
                    mImageView.setImageResource(R.drawable.ic_grande);
                    fab.hide();
                } else if(!closed){
                    fab.show();
                }

                if(position == 1){
                    mPresenter.existsDataFlexExt(programName);
                } else if(position == 2){
                    mPresenter.existsDataAbdAdu(programName);
                } else if(position == 3){
                    mPresenter.existsDataSupPro(programName);
                } else {
                    mImageView.setImageResource(R.drawable.ic_grande);
                }
            }

            @Override
            public void onPageScrollStateChanged(int state) {
            }
        });

        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                int pos = viewPager.getCurrentItem();
                mPresenter.checkDaily(pos, programName);
            }
        });

        fab.hide();

        mPresenter.checkClosed(programName);

    }

    @Override
    public void checkDaily(boolean check, int pos) {
        if(check){
            showNewExerciseDialog(pos);
        } else {
            switch (pos) {
                case 1:
                    showNewFlexExerciseScreen();
                    break;
                case 2:
                    showNewAbdExerciseScreen();
                    break;
                case 3:
                    showNewSupExerciseScreen();
                    break;
                default:
                    break;
            }
        }
    }

    @Override
    public void setTitle(boolean c) {

        if(c){
            closed = true;
            if(getSupportActionBar() != null){
                getSupportActionBar().setTitle(programName + " (Finalizado)");
            }
        } else {
            closed = false;
            if(getSupportActionBar() != null) {
                getSupportActionBar().setTitle(programName);
            }
        }

    }

    @Override
    public void programDeleted(int result) {
        showProgramsScreen(result > 0);
    }

    @Override
    public void programClosed() {
        closed = true;
        Intent intent = getIntent();
        finish();
        startActivity(intent);
    }

    @Override
    public void numSessionsFlexExt(int num) {
        if(num == 0){
            mImageView.setImageResource(R.drawable.emptystate2);
        } else {
            mImageView.setImageResource(R.drawable.ic_grande);
        }
    }

    @Override
    public void numSessionsAbdAdu(int num) {
        if(num == 0){
            mImageView.setImageResource(R.drawable.emptystate2);
        } else {
            mImageView.setImageResource(R.drawable.ic_grande);
        }
    }

    @Override
    public void numSessionsSupPro(int num) {
        if(num == 0){
            mImageView.setImageResource(R.drawable.emptystate2);
        } else {
            mImageView.setImageResource(R.drawable.ic_grande);
        }
    }

    class ViewPagerAdapter extends FragmentPagerAdapter {
        private final List<Fragment> mFragmentList = new ArrayList<>();
        private final List<String> mFragmentTitleList = new ArrayList<>();

        public ViewPagerAdapter(FragmentManager manager) {
            super(manager);
        }

        @Override
        public Fragment getItem(int position) {
            return mFragmentList.get(position);
        }

        @Override
        public int getCount() {
            return mFragmentList.size();
        }

        public void addFragment(Fragment fragment, String title) {
            mFragmentList.add(fragment);
            mFragmentTitleList.add(title);
        }

        @Override
        public CharSequence getPageTitle(int position) {
            return mFragmentTitleList.get(position);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_program_detail, menu);
        if(closed)
        {
            MenuItem item = menu.findItem(R.id.action_close);
            item.setVisible(false);
        }
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        switch (item.getItemId()) {
            case R.id.action_close:
                showCloseDialog();
                break;
            case R.id.action_delete:
                showDeleteDialog();
                break;
            default:
                break;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public boolean onSupportNavigateUp() {
        onBackPressed();
        return true;
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (resultCode >= 1) {
            Intent intent = getIntent();
            finish();
            startActivity(intent);
            switch (resultCode) {
                case FlexExtExerciseActivity.REQUEST_ADD_FLEX:
                    showSuccessfullSavedMessage("Flexión-Extensión");
                    break;
                case AbdAduExerciseActivity.REQUEST_ADD_ABD:
                    showSuccessfullSavedMessage("Abducción-Aducción");
                    break;
                case SupProExerciseActivity.REQUEST_ADD_SUP:
                    showSuccessfullSavedMessage("Supinación-Pronación");
                    break;
            }
        }
    }

    private void showSuccessfullSavedMessage(String ejer) {
        Toast.makeText(this,
                "Ejercicio de "+ ejer +" guardado correctamente", Toast.LENGTH_SHORT).show();
    }

    private void showNewExerciseDialog(int pos) {
        AlertDialog.Builder builder;
        String positiveText, negativeText;
        AlertDialog dialog;
        switch (pos){
            case 1:
                builder = new AlertDialog.Builder(this);
                builder.setTitle(getString(R.string.dialog_title_new_exercise_flex));
                builder.setMessage(getString(R.string.dialog_message_new_exercise));
                positiveText = getString(R.string.ok_continuar);
                builder.setPositiveButton(positiveText,
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                showNewFlexExerciseScreen();
                            }
                        });

                negativeText = getString(R.string.cancel_delete);
                builder.setNegativeButton(negativeText,
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                            }
                        });

                dialog = builder.create();

                dialog.show();
                break;
            case 2:
                builder = new AlertDialog.Builder(this);
                builder.setTitle(getString(R.string.dialog_title_new_exercise_abd));
                builder.setMessage(getString(R.string.dialog_message_new_exercise));
                positiveText = getString(R.string.ok_continuar);
                builder.setPositiveButton(positiveText,
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                showNewAbdExerciseScreen();
                            }
                        });

                negativeText = getString(R.string.cancel_delete);
                builder.setNegativeButton(negativeText,
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                            }
                        });

                dialog = builder.create();

                dialog.show();
                break;
            case 3:
                builder = new AlertDialog.Builder(this);
                builder.setTitle(getString(R.string.dialog_title_new_exercise_sup));
                builder.setMessage(getString(R.string.dialog_message_new_exercise));
                positiveText = getString(R.string.ok_continuar);
                builder.setPositiveButton(positiveText,
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                showNewSupExerciseScreen();
                            }
                        });

                negativeText = getString(R.string.cancel_delete);
                builder.setNegativeButton(negativeText,
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                            }
                        });

                dialog = builder.create();

                dialog.show();
                break;
            default:
                break;
        }

    }

    private void showDeleteDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(getString(R.string.dialog_title_delete));
        builder.setMessage(getString(R.string.dialog_message_delete));

        String positiveText = getString(R.string.ok_delete);
        builder.setPositiveButton(positiveText,
                new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        mPresenter.deleteProgram(programName);
                    }
                });

        String negativeText = getString(R.string.cancel_delete);
        builder.setNegativeButton(negativeText,
                new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                    }
                });

        AlertDialog dialog = builder.create();

        dialog.show();
    }

    private void showCloseDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(getString(R.string.dialog_title_close));
        builder.setMessage(getString(R.string.dialog_message_close));

        String positiveText = getString(R.string.ok_close);
        builder.setPositiveButton(positiveText,
                new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        mPresenter.closeProgram(programName);
                    }
                });

        String negativeText = getString(R.string.cancel_close);
        builder.setNegativeButton(negativeText,
                new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                    }
                });

        AlertDialog dialog = builder.create();

        dialog.show();
    }

    private void showNewFlexExerciseScreen(){

        Intent intent = new Intent(getBaseContext(), FlexExtExerciseActivity.class);
        intent.putExtra(EXTRA_PROGRAM_NAME, programName);
        startActivityForResult(intent, REQUEST_EXERCISE_DONE);
    }

    private void showNewAbdExerciseScreen(){

        Intent intent = new Intent(getBaseContext(), AbdAduExerciseActivity.class);
        intent.putExtra(EXTRA_PROGRAM_NAME, programName);
        startActivityForResult(intent, REQUEST_EXERCISE_DONE);
    }

    private void showNewSupExerciseScreen(){

        Intent intent = new Intent(getBaseContext(), SupProExerciseActivity.class);
        intent.putExtra(EXTRA_PROGRAM_NAME, programName);
        startActivityForResult(intent, REQUEST_EXERCISE_DONE);
    }

    private void showProgramsScreen(boolean requery) {
        if (!requery) {
            showDeleteError();
        }
        setResult(requery ? Activity.RESULT_OK : Activity.RESULT_CANCELED);
        finish();
    }

    private void showDeleteError() {
        Toast.makeText(this,
                "Error al eliminar programa", Toast.LENGTH_SHORT).show();
    }

    private void setupViewPager(ViewPager viewPager) {
        ViewPagerAdapter adapter = new ViewPagerAdapter(getSupportFragmentManager());
        adapter.addFragment(new ProgramDetailFragment(), "Detalles");
        adapter.addFragment(new FlexExtFragment(), "Flex-Ext");
        adapter.addFragment(new AbdAduFragment(), "Abd-Adu");
        adapter.addFragment(new SupProFragment(), "Sup-Pro");
        viewPager.setAdapter(adapter);
    }

}