package com.example.jairo.wristcare.vista;


import android.app.Activity;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.Toast;

import com.example.jairo.wristcare.R;
import com.example.jairo.wristcare.modelo.ProgramsContract.ProgramEntry;
import com.example.jairo.wristcare.modelo.ProgramsDbHelper;
import com.example.jairo.wristcare.presentador.ProgramsFragmentPresenter;
import com.example.jairo.wristcare.util.ProgramsCursorAdapter;

public class ProgramsFragment extends Fragment implements ProgramsView {
    public static final int REQUEST_UPDATE_DELETE_PROGRAM = 2;

    private ListView mProgramsList;
    private ProgramsCursorAdapter mProgramsAdapter;
    private FloatingActionButton mAddButton;

    //Clases patron MVP
    private ProgramsFragmentPresenter mPresenter;
    private ImageView mImageView;


    public ProgramsFragment() {

    }

    public static ProgramsFragment newInstance() {

        return new ProgramsFragment();
    }

    @Override
    public void onCreate(Bundle savedInstanceState){

        super.onCreate(savedInstanceState);
        mPresenter = new ProgramsFragmentPresenter(getActivity(), this);

    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View root = inflater.inflate(R.layout.fragment_programs, container, false);

        // Referencias UI
        mProgramsList = (ListView) root.findViewById(R.id.lawyers_list);
        mProgramsAdapter = new ProgramsCursorAdapter(getActivity(), null);
        mAddButton = (FloatingActionButton) getActivity().findViewById(R.id.fab);
        mImageView = (ImageView) getActivity().findViewById(R.id.imgBackgroundPrograms);
        setHasOptionsMenu(true);

        // Setup
        mProgramsList.setAdapter(mProgramsAdapter);

        // Eventos
        mProgramsList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int i, long l) {

                Cursor currentItem = (Cursor) mProgramsAdapter.getItem(i);
                String currentProgramName = currentItem.getString(currentItem.getColumnIndex(ProgramEntry.NAME));

                showDetailScreen(currentProgramName);
            }
        });
        mAddButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                showAddScreen();
            }
        });

        // Carga de datos
        mPresenter.getAllPrograms();

        return root;
    }

    public void setDataList(Cursor cursor){
        if (cursor != null && cursor.getCount() > 0) {
            mProgramsAdapter.swapCursor(cursor);
            mProgramsList.setAdapter(mProgramsAdapter);
            mImageView.setImageResource(R.drawable.ic_grande);
        } else {
            mProgramsList.setAdapter(null);
            showEmptyState();
            Toast.makeText(getActivity(),
                    "No hay programas", Toast.LENGTH_SHORT).show();
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (Activity.RESULT_OK == resultCode) {
            switch (requestCode) {
                case AddProgramActivity.REQUEST_ADD_PROGRAM:
                    showSuccessfullSavedMessage();
                    //loadPrograms();
                    break;
                case REQUEST_UPDATE_DELETE_PROGRAM:
                    //loadPrograms();
                    break;
            }
        }
        mPresenter.getAllPrograms();
    }

    private void showSuccessfullSavedMessage() {
        Toast.makeText(getActivity(),
                "Programa guardado correctamente", Toast.LENGTH_SHORT).show();
    }

    private void showAddScreen() {
        Intent intent = new Intent(getActivity(), AddProgramActivity.class);
        startActivityForResult(intent, AddProgramActivity.REQUEST_ADD_PROGRAM);
    }

    private void showDetailScreen(String programName) {
        Intent intent = new Intent(getActivity(), ProgramDetailActivity.class);
        intent.putExtra(ProgramsActivity.EXTRA_PROGRAM_NAME, programName);
        startActivityForResult(intent, REQUEST_UPDATE_DELETE_PROGRAM);
    }

    private void showEmptyState(){

        mImageView.setImageResource(R.drawable.emptystate1);
    }
}