package com.example.jairo.wristcare.vista;

import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.Spinner;

import com.example.jairo.wristcare.R;
import com.example.jairo.wristcare.presentador.SupProFragmentPresenter;
import com.github.mikephil.charting.charts.LineChart;
import com.github.mikephil.charting.components.AxisBase;
import com.github.mikephil.charting.components.Description;
import com.github.mikephil.charting.components.Legend;
import com.github.mikephil.charting.components.XAxis;
import com.github.mikephil.charting.components.YAxis;
import com.github.mikephil.charting.data.LineData;
import com.github.mikephil.charting.formatter.IAxisValueFormatter;
import com.github.mikephil.charting.interfaces.datasets.ILineDataSet;

import java.util.ArrayList;


public class SupProFragment extends Fragment implements SupProFragmentView {

    public static boolean datos;

    private SupProFragmentPresenter mPresenter;

    private String mProgramName;

    private ImageView mImageView;
    private Spinner spinner;
    private LineChart mChart;

    public SupProFragment(){

    }

    @Override
    public void onCreate(Bundle savedInstanceState){
        super.onCreate(savedInstanceState);

        mPresenter = new SupProFragmentPresenter(getActivity(), this);
        mProgramName = ((ProgramDetailActivity)getActivity()).programName;

    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View root = inflater.inflate(R.layout.fragment_sup_pro, container, false);

        spinner = (Spinner) root.findViewById(R.id.spinner_sup_pro);
        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(getActivity(), R.array.spinner_sup_pro, android.R.layout.simple_spinner_dropdown_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spinner.setAdapter(adapter);
        spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                mPresenter.setDataChart(mProgramName, mChart, position);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        mImageView = (ImageView) getActivity().findViewById(R.id.imgBackgroundDetail);
        mChart = (LineChart) root.findViewById(R.id.chartSup);

        mPresenter.getAxis(mProgramName);

        return root;
    }

    @Override
    public void setDataChart(ArrayList<ILineDataSet> lineDataSets, Description desc) {
        if (lineDataSets.isEmpty()){
            datos = false;
            spinner.setVisibility(View.GONE);
            mChart.setVisibility(View.GONE);
        } else {
            datos = true;
            mChart.setDescription(desc);
            mChart.setData(new LineData(lineDataSets));
            mChart.invalidate();
        }
    }

    @Override
    public void setAxisStyle(String[] dateStrings) {

        final String[] date = dateStrings;

        if(dateStrings.length > 0) {
            IAxisValueFormatter formatter = new IAxisValueFormatter() {
                @Override
                public String getFormattedValue(float value, AxisBase axis) {
                    return date[(int) value];
                }
            };

            IAxisValueFormatter formatterSingleValue = new IAxisValueFormatter() {
                @Override
                public String getFormattedValue(float value, AxisBase axis) {
                    if (((int) value) < 0) {
                        return "";
                    } else if ((int) value > 0) {
                        return "";
                    } else {
                        return date[(int) value];
                    }
                }
            };

            IAxisValueFormatter formatDegree = new IAxisValueFormatter() {
                @Override
                public String getFormattedValue(float value, AxisBase axis) {
                    return String.valueOf((int) value) + " º";
                }
            };

            mChart.getLegend().setTextSize(15f);
            mChart.getLegend().setForm(Legend.LegendForm.CIRCLE);
            mChart.getLegend().setFormSize(10f);
            mChart.getLegend().setXEntrySpace(10f);

            XAxis xAxisFecha = mChart.getXAxis();
            xAxisFecha.setGranularity(1f);
            if (date.length > 1) {
                xAxisFecha.setValueFormatter(formatter);
            } else {
                xAxisFecha.setValueFormatter(formatterSingleValue);
            }

            xAxisFecha.setPosition(XAxis.XAxisPosition.BOTTOM);
            xAxisFecha.setDrawGridLines(false);
            xAxisFecha.setLabelRotationAngle(-45);

            YAxis yAxisL = mChart.getAxisLeft();
            YAxis yAxisR = mChart.getAxisRight();

            yAxisR.setEnabled(false);
            yAxisL.setGranularity(2f);
            yAxisL.setValueFormatter(formatDegree);
            yAxisL.setAxisMinimum(0);
        }
    }
}